function A = subsasgn(A,S,B)

% A = SUBSASGN(A,S,B)               Jeffrey J. DaCunha 9/02/2003
%                                   Ian A. Gravagne 12/27/2003
%
% A(n) = 2          Will assign the nth element of the timescale's data object to 2, where n
%                   must be a scalar within the length of the timescale data array. The timescale
%                   may not shrink or expand under this operation.
% A(2:4) = [3:5]    Returns an error. 
% A(2:4,k) = [3:5]  Will assign elements 2 through 4 of interval k the values [3,4,5]. The interval
%                   may shrink or expand under this operation, but more intervals may not be added. 
%                   Assignment of the empty interval A(:,n) = [] will not delete the interval but
%                   will simply leave its contents empty. 
% 
% A{n} = T          Embeds timescale T as interval n in timescale A. The assignment requires T to
%                   have only one interval. Intervals may be deleted by using the syntax A{n} = [].
%                   If n is greater than the number of intervals in A, then A will expand to have n
%                   intervals by filling in any missing intervals with empty intervals of type 'd'. 
%

%%%% PRELIMINARY ERROR CHECKS %%%%
if(size(S,2) >1)
    error('timescale object cannot be multiply dereferenced.');
elseif(length(S.subs) > 2)
    error('timescale object only has 2 dimensions (data and interval)');
end

%%%% IF ASSIGMENT IS () %%%%
if(isequal(S.type,'()') )
    
    % Error checks for () 
    if(length(S.subs) > 2)
        error('Timescale assigment must be of form A(:,k)= or A(m:n,k)= ... ');
    elseif( ~isnumeric(B) )
        error('In the timescale assignemtn A(n:m,k) = B, B must be a vector');
    end

    % Assignment handler for ()
    if(~isempty(B))
        B = reshape(B,1,prod(size(B)));
    end
    
    if(length(S.subs) == 1)             
        if(S.subs{1} == ':')                      % User entered A(:)
            error('cannot assign the entire timescale. Use A(:,k) to assign an entire interval, or use the timescale constructor.');
        elseif(length(S.subs{1}) ~= [1,1])        % User entered A(3:5)
            error('cannot assign a subset of the timescale. Use A(n:m,k) to assign a subset of interval k, or use the timescale constructor.');
        elseif(S.subs{1} > length(A.data))
            error('In timescale assignment A(n) = 5, n must be within length(A)');
        else
            A.data(S.subs{1}) = B;
        end
        
    else                                    % User entered A(2,2) or A(4:6,3)
    
        type = [0,A.type(1,:)];
        
        if( (S.subs{2} == ':') | (length(S.subs{2}) > 1) )
            error('timescale object can only have integer dereference for 2nd dimension.');
        elseif(S.subs{2} > size(A.type,2))
            message = ['timescale object does not have ',num2str(S.subs{2}),' intervals.'];
            error(message);
        else
            p = type(S.subs{2}) + 1;
            q = type(S.subs{2}+1);
            interval = A.data(p:q);
            oldsize = length(interval);
            if(S.subs{1} == ':')
                interval = B;
            else
                    S.subs{1};
                    B;
                interval(S.subs{1}) = B;
            end
            newsize = length(interval);
            A.data = [A.data(1:p-1),interval,A.data(q+1:length(A.data))];
            sizediff = newsize - oldsize;
            A.type(1,S.subs{2}:size(A.type,2)) = A.type(1,S.subs{2}:size(A.type,2)) + sizediff;
        end
        
    end
    
%%%% IF ASSIGNMENT IS {} %%%%
elseif(isequal(S.type,'{}'))
    
    % Error checks for {}
    if( (length(S.subs) > 1) | (S.subs{1} == ':') | length(S.subs{1}) > 1 )
        error('In the timescale assignment A{n}=B, n must be an integer.');
    elseif( ~isa(B,'timescale') & ~isempty(B) )
        error('In the timescale assignment A{n}=B, B must be a timescale object.');
    end
        
    % Assignment handler for {}
    
    intnum = S.subs{1};
    lastintnum = size(A.type,2);
    lastindex = length(A.data);
    
    if(intnum <= size(A.type,2));
        
        if( ~isempty(B) )
            type = [0,A.type(1,:)];
            p = type(intnum) + 1;
            q = type(intnum+1);
            interval = A.data(p:q);
            oldsize = length(interval);
            interval = B.data;
            newsize = length(interval);
            A.data = [A.data(1:p-1),interval,A.data(q+1:lastindex)];
            sizediff = newsize - oldsize;
            A.type(1,intnum:lastintnum) = A.type(1,intnum:lastintnum) + sizediff;    
            A.type(2,intnum) = B.type(2,1);
        else
            type = [0,A.type(1,:)];
            p = type(intnum) + 1;
            q = type(intnum+1);
            interval = A.data(p:q);
            oldsize = length(interval);
            A.data = [A.data(1:p-1),A.data(q+1:lastindex)];
            sizediff = -oldsize;
            A.type(1,intnum:lastintnum) = A.type(1,intnum:lastintnum) + sizediff;           
            A.type(:,intnum) = [];
        end
        
    else
        
        if( ~isempty(B) )
            A.data = [A.data, B.data];
            A.type = [A.type, [lastindex*ones(1,intnum-lastintnum);ones(1,intnum-lastintnum)]];
            A.type(1,intnum) = length(A.data);
            A.type(2,intnum) = B.type(2,1);
        else
            A.type = [A.type, [lastindex*ones(1,intnum-lastintnum-1);ones(1,intnum-lastintnum-1)]];
        end
            
    end    
        
        
    
%%%% IF ASSIGNMENT IS 'dot' %%%%
elseif(isequal(S(1).type,'.'))
    
    error('Timescale object fields cannot be assigned directly.');
    
%%%% ALL OTHER CASES %%%%
else
    
    error('Timescale assigment could be interpreted.');
    
end
